import os.path
import struct
import sys

with open("_header.bin", "wb") as f:
    x = struct.pack('<H', os.path.getsize(sys.argv[1] + '_dfu.bin') // 512 + 1)
    y = [0xda, 0xff, x[0], x[1], 0xff, 0xff, 0xff, 0xff]
    f.write(bytearray(y))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Jared Boone
#
# This file is part of HackRF.
#
# This is a free hardware design; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#  
# This design is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# You should have received a copy of the GNU General Public License
# along with this design; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#

# This program configures a Silicon Labs Si5351C clock generation IC
# via an I2C interface, using a Dangerous Prototypes Bus Pirate v3.5
# and the pyBusPirateLite Python library.
#
# Si5351C:
#   http://www.silabs.com/products/clocksoscillators/clock-generators-and-buffers/Pages/clock+vcxo.aspx
# Bus Pirate:
#   http://dangerousprototypes.com/docs/Bus_Pirate
# pyBusPirateLite:
#   http://code.google.com/p/the-bus-pirate/
#
# This Si5351C configuration is a work in progress, it does not yet
# implement the clock plan described in the HackRF wiki:
# https://github.com/mossmann/hackrf/wiki/Clocking
#
# To follow progress in configuring the Si5351C, please refer to the
# HackRF wiki:
# https://github.com/mossmann/hackrf/wiki/Lemondrop-Bring-Up

import sys

from pyBusPirateLite.I2C import *
from math import log

if len(sys.argv) < 2:
    print('Usage: %s <path to serial device>' % (sys.argv[0],))
    sys.exit()

serial_path = sys.argv[1]
print('Connecting to Bus Pirate on "%s"...' % (serial_path,))

i2c = I2C(serial_path, 115200)
i2c.resetBP()

print('Entering I2C mode...')
if not i2c.BBmode():
    print('BBmode() failed')
    sys.exit()

if not i2c.enter_I2C():
    print('enter_I2C() failed')
    sys.exit()

if not i2c.set_speed(I2CSpeed._400KHZ):
    print('set_speed() failed')
    sys.exit()

i2c.cfg_pins(I2CPins.POWER)
i2c.timeout(0.2)

def write_registers(first_register_number, values):
    data = [0xC0, first_register_number]
    if isinstance(values, (list, tuple)):
        data.extend(values)
    else:
        data.append(values)
    i2c.send_start_bit()
    i2c.bulk_trans(len(data), data)
    i2c.send_stop_bit()

# r is the R output divider (should be 1, 2, 4, 8. . .)
# note: si5351c.c's r parameter is encoded (log not taken)
# use p2=0 and p3=1 for integer mode
def set_multisynth_parameters(ms_n, p1, p2, p3, r):
    register_number = 42 + (ms_n * 8)
    values = (
        (p3 >> 8) & 0xFF,
        (p3 >> 0) & 0xFF,
        (int(log(r, 2)) << 4) | (0 << 2) | ((p1 >> 16) & 0x3),
        (p1 >> 8) & 0xFF,
        (p1 >> 0) & 0xFF,
        (((p3 >> 16) & 0xF) << 4) | (((p2 >> 16) & 0xF) << 0),
        (p2 >> 8) & 0xFF,
        (p2 >> 0) & 0xFF
    )
    write_registers(register_number, values)

# Get the appropriate P1 setting for a given frequency.
# Assumes VCO is 800 MHz and you want integer division and R=4.
def integer_p1(frequency):
	return int(800e6/frequency) * 128 - 512

def set_codec_rate(frequency):
    set_multisynth_parameters(1, integer_p1(frequency * 4), 0, 1, 4)

print('Configuring Si5351...')

# Disable all CLKx outputs.
write_registers(3, 0xFF)

# Turn off OEB pin control for all CLKx
write_registers(9, 0xFF)

# Power down all CLKx
write_registers(16, (0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0xC0, 0xC0))

# Register 183: Crystal Internal Load Capacitance
# Reads as 0xE4 on power-up
# Set to 10pF (until I find out what loading the crystal/PCB likes best)
write_registers(183, 0xE4)

# Register 187: Fanout Enable
# Turn on XO and MultiSynth fanout only.
write_registers(187, 0x50)

# Register 15: PLL Input Source
# CLKIN_DIV=0 (Divide by 1)
# PLLB_SRC=0 (XTAL input)
# PLLA_SRC=0 (XTAL input)
write_registers(15, 0x00)

# MultiSynth NA (PLL1)
write_registers(26, (0x00, 0x01, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00))

# MultiSynth NB (PLL2)
###

# MultiSynth 0
# This is the source for the MAX2837 clock input.
# It is also used to generate the ADC/DAC clocks.
set_multisynth_parameters(0, integer_p1(80e6), 0, 1, 2) # 40MHz with R=2

# MultiSynth 1 (MAX5864 and CPLD)
#set_codec_rate(20e6)
set_multisynth_parameters(1, integer_p1(80e6), 0, 1, 4) # 20MHz with R=2

# MultiSynth 2 (CPLD)
set_multisynth_parameters(2, integer_p1(80e6), 0, 1, 1) # 20MHz with R=2

# MultiSynth 3 (CPLD)
set_multisynth_parameters(3, integer_p1(80e6), 0, 1, 1) # 20MHz with R=2

# MultiSynth 4
# This is the source for the LPC43xx external clock input.
set_multisynth_parameters(4, 8021, 1, 3, 1)    # 12MHz
#set_multisynth_parameters(4, integer_p1(20e6), 0, 1, 1) # 20 MHz
#set_multisynth_parameters(4, integer_p1(80e6), 0, 1, 4) # 20 MHz using R=4
#set_multisynth_parameters(4, 3584, 0, 1, 1)    # 25MHz

# MultiSynth 6/7 R dividers
write_registers(92, 0x00)

# Registers 16 through 23: CLKx Control
# CLK0:
#   CLK0_PDN=0 (powered up)
#   MS0_INT=1 (integer mode)
#   MS0_SRC=0 (PLLA as source for MultiSynth 0)
#   CLK0_INV=0 (not inverted)
#   CLK0_SRC=3 (MS0 as input source)
#   CLK0_IDRV=3 (8mA)
# CLK1:
#   CLK1_PDN=0 (powered up)
#   MS1_INT=1 (integer mode)
#   MS1_SRC=0 (PLLA as source for MultiSynth 1)
#   CLK1_INV=0 (not inverted)
#   CLK1_SRC=2 (MS0 as input source)
#   CLK1_IDRV=3 (8mA)
# CLK2:
#   CLK2_PDN=0 (powered up)
#   MS2_INT=1 (integer mode)
#   MS2_SRC=0 (PLLA as source for MultiSynth 1)
#   CLK2_INV=0 (not inverted)
#   CLK2_SRC=2 (MS0 as input source)
#   CLK2_IDRV=3 (8mA)
# CLK3:
#   CLK3_PDN=0 (powered up)
#   MS3_INT=1 (integer mode)
#   MS3_SRC=0 (PLLA as source for MultiSynth 1)
#   CLK3_INV=0 (not inverted)
#   CLK3_SRC=2 (MS0 as input source)
#   CLK3_IDRV=3 (8mA)
# CLK4:
#   CLK4_PDN=0 (powered up)
#   MS4_INT=0 (fractional mode -- to support 12MHz to LPC for USB DFU)
#   MS4_SRC=0 (PLLA as source for MultiSynth 4)
#   CLK4_INV=0 (not inverted)
#   CLK4_SRC=3 (MS4 as input source)
#   CLK4_IDRV=3 (8mA)
write_registers(16, (0x4F, 0x4B, 0x4B, 0x4B, 0x0F, 0x80, 0xC0, 0xC0))

# Enable CLK outputs 0, 1, 4 only.
write_registers(3, 0xFF ^ 0b00011111)

raw_input("<return> to quit...")

print('Powering down...')
i2c.cfg_pins(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 Jared Boone
#
# This file is part of HackRF.
#
# This is a free hardware design; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#  
# This design is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#  
# You should have received a copy of the GNU General Public License
# along with this design; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#

# This program is used to verify the logic on the Lollipop board, to
# make sure control of the various RF paths is correct.

class Component(object):
    def __init__(self, **kwargs):
        self.state = kwargs
        
    def __repr__(self):
        state_key = ''.join(
            map(str,
                map(int,
                    (self.state[input] for input in self.inputs)
                )
            )
        )
        if state_key not in self.states:
            return 'Invalid'
        else:
            return self.functions[self.states[state_key]]

class SKY13317(Component):
    inputs = (
        'V1',
        'V2',
        'V3',
    )
    
    states = {
        # V1, V2, V3
        '100': 'RFC to RF1',
        '010': 'RFC to RF2',
        '001': 'RFC to RF3',
    }

class SKY13351(Component):
    inputs = (
        'VCTL1',
        'VCTL2',
    )
    
    states = {
        # VCTL1, VCTL2
        '01': 'INPUT to OUTPUT1',
        '10': 'INPUT to OUTPUT2',
    }

class U2_4(SKY13351):
    name = 'U2/4'
    functions = {
        'INPUT to OUTPUT1': 'tx bandpass',
        'INPUT to OUTPUT2': 'tx mixer'
    }

class U6_9(SKY13351):
    name = 'U6/9'
    functions = {
        'INPUT to OUTPUT1': 'tx lowpass',
        'INPUT to OUTPUT2': 'tx highpass',
    }

class U3(SKY13317):
    name = 'U3'
    functions = {
        'RFC to RF1': 'tx highpass',
        'RFC to RF2': 'tx lowpass',
        'RFC to RF3': 'tx bandpass',
    }

class U7(SKY13351):
    name = 'U7'
    functions = {
        'INPUT to OUTPUT1': 'rx switch',
        'INPUT to OUTPUT2': 'tx path',
    }

class U10(SKY13351):
    name = 'U10'
    functions = {
        'INPUT to OUTPUT1': 'tx/rx switch',
        'INPUT to OUTPUT2': 'rx antenna',
    }

class U15(SKY13317):
    name = 'U15'
    functions = {
        'RFC to RF1': 'rx bandpass',
        'RFC to RF2': 'rx highpass',
        'RFC to RF3': 'rx lowpass',
    }

class U12_14(SKY13351):
    name = 'U12/14'
    functions = {
        'INPUT to OUTPUT1': 'rx lowpass',
        'INPUT to OUTPUT2': 'rx highpass',
    }

class U16_18(SKY13351):
    name = 'U16/18'
    functions = {
        'INPUT to OUTPUT1': 'rx mixer',
        'INPUT to OUTPUT2': 'rx bandpass',
    }

def compute_logic(**inputs):
    outputs = dict(inputs)
    outputs['swtxb2'] = not inputs['swtxb1']
    outputs['swrxb2'] = not inputs['swrxb1']
    outputs['swtxa2'] = not inputs['swtxa1']
    outputs['swrxa2'] = not inputs['swrxa1']
    outputs['swd2'] = not inputs['swd1']
    outputs['swrxv2'] = outputs['swrxb2'] and outputs['swrxa1']
    outputs['swrxv3'] = outputs['swrxb2'] and outputs['swrxa2']
    outputs['swtxv1'] = outputs['swtxa1'] and outputs['swtxb1']
    outputs['swtxv2'] = outputs['swtxa2'] and outputs['swtxb1']
    
    # Force boolean True/False (result of "not" operator) to 1 or 0.
    for key in outputs:
        outputs[key] = int(outputs[key])
    
    return outputs

def print_signals(signals):
    print(', '.join(('%s=%s' % (name, signals[name]) for name in sorted(signals))))

def print_circuit_state(signals):
    components = (
        U2_4(VCTL1=signals['swtxb1'], VCTL2=signals['swtxb2']),
        U6_9(VCTL1=signals['swtxa1'], VCTL2=signals['swtxa2']),
        U3(V1=signals['swtxv1'], V2=signals['swtxv2'], V3=signals['swtxb2']),
        U7(VCTL1=signals['swd2'], VCTL2=signals['swd1']),
        U10(VCTL1=signals['swd2'], VCTL2=signals['swd1']),
        U15(V1=signals['swrxb1'], V2=signals['swrxv2'], V3=signals['swrxv3']),
        U12_14(VCTL1=signals['swrxa1'], VCTL2=signals['swrxa2']),
        U16_18(VCTL1=signals['swrxb1'], VCTL2=signals['swrxb2'])
    )
    
    for component in components:
        print('%s: %s' % (component.name, component))

def make_bits_from_numbers(i, bit_count):
    return [int(c) for c in bin(i)[2:].zfill(bit_count)]

print('Transmit')
print('========')
print
for i in range(4):
    inputs = {
        'swtxb1': (i >> 1) & 1,
        'swtxa1': (i >> 0) & 1,
        'swrxa1': 0,
        'swrxb1': 0,
        'swd1': 0,
    }
    
    outputs = compute_logic(**inputs)
    print_signals(outputs)
    print_circuit_state(outputs)
    
    print

print('Receive')
print('========')
print
for i in range(4):
    inputs = {
        'swtxb1': 0,
        'swtxa1': 0,
        'swrxa1': (i >> 1) & 1,
        'swrxb1': (i >> 0) & 1,
        'swd1': 0,
    }
    
    outputs = compute_logic(**inputs)
    print_signals(outputs)
    print_circuit_state(outputs)
    
    print



#!/usr/bin/env python

# Copyright 2013 Jared Boone
#
# Display the LPC43xx Clock Generation Unit (CGU) registers in an
# easy-to-read format.
#
# This file is part of HackRF.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.

"""
In GDB:
	dump binary memory cgu.bin 0x40050014 0x400500cc
"""

import sys
from struct import unpack

address = 0x40050014

f = open(sys.argv[1], 'read')
d = '\x00' * 20 + f.read()
length = len(d)
f.close()

def print_data(d):
	for i in range(0, length, 16):
		values = unpack('<IIII', d[i:i+16])
		values = ['%08x' % v for v in values]
		values_str = ' '.join(values)
		line = '%08x: %s' % (address + i, values_str)
		print(line)

#print_data(d)
#sys.exit(0)

data = {}
for i in range(0, length, 4):
	data[i] = unpack('<I', d[i:i+4])[0]

registers = {
	0x14: {
		'name': 'FREQ_MON',
		'fields': (
			('RCNT', 0, 9),
			('FCNT', 9, 14),
			('MEAS', 23, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x18: {
		'name': 'XTAL_OSC_CONTROL',
		'fields': (
			('ENABLE', 0, 1),
			('BYPASS', 1, 1),
			('HF', 2, 1)
		),
	},
	0x1c: {
		'name': 'PLL0USB_STAT',
		'fields': (
			('LOCK', 0, 1),
			('FR', 1, 1),
		),
	},
	0x20: {
		'name': 'PLL0USB_CTRL',
		'fields': (
			('PD', 0, 1),
			('BYPASS', 1, 1),
			('DIRECTI', 2, 1),
			('DIRECTO', 3, 1),
			('CLKEN', 4, 1),
			('FRM', 6, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x24: {
		'name': 'PLL0USB_MDIV',
		'fields': (
			('MDEC', 0, 17),
			('SELP', 17, 5),
			('SELI', 22, 6),
			('SELR', 28, 4),
		),
	},
	0x28: {
		'name': 'PLL0USB_NP_DIV',
		'fields': (
			('PDEC', 0, 7),
			('NDEC', 12, 10),
		),
	},
	0x40: {
		'name': 'PLL1_STAT',
		'fields': (
			('LOCK', 0, 1),
		),
	},
	0x44: {
		'name': 'PLL1_CTRL',
		'fields': (
			('PD', 0, 1),
			('BYPASS', 1, 1),
			('FBSEL', 6, 1),
			('DIRECT', 7, 1),
			('PSEL', 8, 2),
			('AUTOBLOCK', 11, 1),
			('NSEL', 12, 2),
			('MSEL', 16, 8),
			('CLK_SEL', 24, 5),
		),
	},
	0x48: {
		'name': 'IDIVA_CTRL',
		'fields': (
			('PD', 0, 1),
			('IDIV', 2, 2),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x4C: {
		'name': 'IDIVB_CTRL',
		'fields': (
			('PD', 0, 1),
			('IDIV', 2, 4),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x50: {
		'name': 'IDIVC_CTRL',
		'fields': (
			('PD', 0, 1),
			('IDIV', 2, 4),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x54: {
		'name': 'IDIVD_CTRL',
		'fields': (
			('PD', 0, 1),
			('IDIV', 2, 4),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x58: {
		'name': 'IDIVE_CTRL',
		'fields': (
			('PD', 0, 1),
			('IDIV', 2, 8),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x5C: {
		'name': 'BASE_SAFE_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x60: {
		'name': 'BASE_USB0_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x64: {
		'name': 'BASE_PERIPH_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x68: {
		'name': 'BASE_USB1_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x6C: {
		'name': 'BASE_M4_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x70: {
		'name': 'BASE_SPIFI_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x74: {
		'name': 'BASE_SPI_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x78: {
		'name': 'BASE_PHY_RX_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x7C: {
		'name': 'BASE_PHY_TX_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x80: {
		'name': 'BASE_APB1_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x84: {
		'name': 'BASE_APB3_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x88: {
		'name': 'BASE_LCD_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x8C: {
		'name': 'BASE_VADC_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x90: {
		'name': 'BASE_SDIO_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x94: {
		'name': 'BASE_SSP0_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x98: {
		'name': 'BASE_SSP1_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0x9C: {
		'name': 'BASE_UART0_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0xA0: {
		'name': 'BASE_UART1_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0xA4: {
		'name': 'BASE_UART2_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0xA8: {
		'name': 'BASE_UART3_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0xAC: {
		'name': 'BASE_OUT_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0xC0: {
		'name': 'BASE_APLL_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0xC4: {
		'name': 'BASE_CGU_OUT0_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	0xC8: {
		'name': 'BASE_CGU_OUT1_CLK',
		'fields': (
			('PD', 0, 1),
			('AUTOBLOCK', 11, 1),
			('CLK_SEL', 24, 5),
		),
	},
	# TODO: Add other CGU registers. I did the ones that were
	# valuable to me to debug CPU clock issues.
}

for address in sorted(registers):
	register = registers[address]
	name = register['name']
	fields = register['fields']
	value = data[address]
	bits = bin(value)[2:].zfill(32)
	print('%03x %20s %s = %08x' % (address, name, bits, value))
	for field in fields:
		name, low_bit, count = field
		field_value = (value >> low_bit) & ((1 << count) - 1)
		field_bits = bin(field_value)[2:].zfill(count) + ' ' * low_bit
		field_bits = field_bits.rjust(32)
		print('%03s %20s %s = %8x %s' % ('', '', field_bits, field_value, name))



#!/usr/bin/env python

# Copyright 2013 Jared Boone
#
# Interpret the LPC43xx Clock Generation Unit (CGU) FREQ_MON register
# and display the estimated clock frequency.
#
# This file is part of HackRF.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.

"""
Inside GDB:
	(gdb) set {int}0x40050014 = 0x09800000
	(gdb) x 0x40050014
	0x40050014:     0x091bd000
This script:
	./check_clock.py 0x<register value> 0x<RCNT initial value>
"""

import sys

clock_source_names = {
	0x00: '32 kHz oscillator',
	0x01: 'IRC',
	0x02: 'ENET_RX_CLK',
	0x03: 'ENET_TX_CLK',
	0x04: 'GP_CLKIN',
	0x06: 'Crystal oscillator',
	0x07: 'PLL0USB',
	0x08: 'PLL0AUDIO',
	0x09: 'PLL1',
	0x0c: 'IDIVA',
	0x0d: 'IDIVB',
	0x0e: 'IDIVC',
	0x0f: 'IDIVD',
	0x10: 'IDIVE',
}

reg_value = int(sys.argv[1], 16)
rcnt = int(sys.argv[2], 16)

print('0x%08x' % reg_value)

rcnt_final = reg_value & 0x1ff
fcnt = (reg_value >> 9) & 0x3fff
clock_source = (reg_value >> 24) & 0x1f
fref = 12e6

if rcnt_final != 0:
	raise RuntimeError('RCNT did not reach 0')

print('RCNT: %d' % rcnt)
print('FCNT: %d' % fcnt)
print('Fref: %d' % fref)

clock_hz = fcnt / float(rcnt) * fref
clock_mhz = clock_hz / 1e6
clock_name = clock_source_names[clock_source] if clock_source in clock_source_names else 'Reserved'
print('%s: %.3f MHz' % (clock_name, clock_mhz))
